<?php

namespace Drupal\image_sizes_extras\Plugin\Field\FieldFormatter;

use Drupal\Core\Field\FieldItemListInterface;
use Drupal\Core\Field\Plugin\Field\FieldType\EntityReferenceItem;
use Drupal\Core\Form\FormStateInterface;
use Drupal\image\Plugin\Field\FieldType\ImageItem;
use Drupal\image_sizes\Plugin\Field\FieldFormatter\ImageSizesPresetFormatter;
use Drupal\media\MediaInterface;

/**
 * Plugin implementation of the 'Example' formatter.
 *
 * @FieldFormatter(
 *   id = "image_sizes_extras_formatter",
 *   label = @Translation("Images sizes extras formatter"),
 *   field_types = {
 *     "image",
 *     "entity_reference"
 *   }
 * )
 */
class ImageSizesExtrasFormatter extends ImageSizesPresetFormatter {

  /**
   * {@inheritdoc}
   */
  public function settingsForm(array $form, FormStateInterface $formState) {
    $element = parent::settingsForm($form, $formState);

    unset($element["load_invisible"]);

    return $element;
  }

  /**
   * {@inheritdoc}
   */
  public function viewElements(FieldItemListInterface $items, $langcode) {
    $elements = parent::viewElements($items, $langcode);

    foreach ($items as $delta => $item) {
      $elements[$delta]['#theme'] = 'image_sizes_extras';

      if ($item instanceof ImageItem) {
        $alt = $item->alt;
        $title = $item->title;
      }

      if ($item instanceof EntityReferenceItem && $item->entity instanceof MediaInterface) {
        $field = $item->entity->getSource()->getSourceFieldDefinition($item->entity->bundle->entity);
        if (isset($field) && !$item->entity->get($field->getName())->isEmpty() && $field->getType() == 'image') {
          $alt = $item->entity->get($field->getName())->first()->alt;
          $title = $item->entity->get($field->getName())->first()->title;
        }
      }

      $alt && $elements[$delta]['#attributes']['alt'] = $alt;
      $title && $elements[$delta]['#attributes']['title'] = $title;
    }

    return $elements;
  }
}
